#!/usr/bin/python
#
# main.py
# Copyright (C) Moe 2011 <moe@diccuric.org>
# 
# 1d-Pacman is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# 1d-Pacman is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import pygame, sys,os, random, math, time
from pygame.locals import *

class Player:
	def __init__(self, width):
		self.pos = 0
		self.graphic_right = pygame.image.load("flatman.png")
		self.graphic_left = pygame.transform.flip(self.graphic_right, True, False)
		self.graphic_right_closed = pygame.image.load("flatman_closed.png")
		self.graphic_left_closed = pygame.transform.flip(self.graphic_right_closed, True, False)
		self.dx = 0
		self.width = width - 32
		self.alive = True
		self.graphic = self.graphic_right
		self.graphic_closed = self.graphic_right_closed
		self.closed = False
		self.closed_diff = 0

	def draw(self, screen):
		if self.alive:
			if self.closed:
				screen.blit(self.graphic_closed, [self.pos, 175])
			else:
				screen.blit(self.graphic, [self.pos, 175])

	def setDx(self, dx):
		if dx < 0:
			self.graphic = self.graphic_left
			self.graphic_closed = self.graphic_left_closed
		elif dx > 0:
			self.graphic = self.graphic_right
			self.graphic_closed = self.graphic_right_closed
		self.dx = dx

	def update(self, diff):
		if self.alive:
			if (self.dx < 0 and self.pos > 0) or (self.dx > 0 and self.pos < self.width):
				self.pos = self.pos + 250 * diff * self.dx
			self.closed_diff = self.closed_diff + 1000 * diff
			if self.closed_diff > 250:
				self.closed_diff = 0
				self.closed = not self.closed
				

	def getPos(self):
		return int(self.pos)

	def kill(self):
		self.alive = False

class Enemy:
	def __init__(self, pos, width):
		self.pos = 32*pos
		self.graphics_evil_right = pygame.image.load("ghost_evil.png")
		self.graphics_evil_closed_right = pygame.image.load("ghost_evil_closed.png")
		self.graphics_eatable_right = pygame.image.load("ghost_eatable.png")
		self.graphics_eatable_closed_right = pygame.image.load("ghost_eatable_closed.png")
		
		self.graphics_evil_left = pygame.transform.flip(self.graphics_evil_right, True, False)
		self.graphics_evil_closed_left = pygame.transform.flip(self.graphics_evil_closed_right, True, False)
		self.graphics_eatable_left = pygame.transform.flip(self.graphics_eatable_right, True, False)
		self.graphics_eatable_closed_left = pygame.transform.flip(self.graphics_eatable_closed_right, True, False)
		
		self.graphics_evil = self.graphics_evil_right
		self.graphics_evil_closed = self.graphics_evil_closed_right
		self.graphics_eatable = self.graphics_eatable_right
		self.graphics_eatable_closed = self.graphics_eatable_closed_right
		self.eatable = False
		self.dx = 0
		self.width = width
		self.alive = True
		self.closed = False
		self.closed_diff = 0
		self.turnCount = 0
		self.animSpeed = random.randint(350, 750)
		self.walkSpeed = random.randint(50, 250)
		
	def draw(self, screen):
		if self.alive:
			if self.eatable:
				if self.closed:
					screen.blit(self.graphics_eatable_closed, [self.pos, 175])
				else:
					screen.blit(self.graphics_eatable, [self.pos, 175])
			else:
				if self.closed:
					screen.blit(self.graphics_evil_closed, [self.pos, 175])
				else:
					screen.blit(self.graphics_evil, [self.pos, 175])

	def update(self, diff):
		if self.alive:
			self.turnCount = self.turnCount + 1
			if self.turnCount > 5:
				if self.dx < 0:
					self.dx = random.randint(-10, 1)
				elif self.dx > 0:
					self.dx = random.randint(-1, 10)
				else:
					self.dx = random.randint(-1, 1)
				self.turnCount = 0

			if self.dx < 0:
				self.dx = -1
			elif self.dx > 0:
				self.dx = 1
				
			if self.dx < 0:
				self.graphics_evil = self.graphics_evil_left
				self.graphics_evil_closed = self.graphics_evil_closed_left
				self.graphics_eatable = self.graphics_eatable_left
				self.graphics_eatable_closed = self.graphics_eatable_closed_left
			elif self.dx > 0:
				self.graphics_evil = self.graphics_evil_right
				self.graphics_evil_closed = self.graphics_evil_closed_right
				self.graphics_eatable = self.graphics_eatable_right
				self.graphics_eatable_closed = self.graphics_eatable_closed_right
			
			if (self.dx < 0 and self.pos > 0) or (self.dx > 0 and self.pos < self.width):
				self.pos = self.pos + self.walkSpeed * diff * self.dx
			if self.eatable:
				self.count = self.count + int(1000 * diff)
				if self.count >= 1250:
					self.eatable = False
			self.closed_diff = self.closed_diff + 1000 * diff
			if self.closed_diff > self.animSpeed:
				self.closed_diff = 0
				self.closed = not self.closed

	def setEatable(self):
		self.eatable = True
		self.count = 0

	def isEatable(self):
		return self.eatable

	def getPos(self):
		return self.pos

	def setEaten(self):
		self.alive = False

	def isAlive(self):
		return self.alive

class Flatman:
	def __init__(self):
		pygame.init()
		self.width = 1000
		self.height = 250
		window = pygame.display.set_mode((self.width, self.height)) 
		pygame.display.set_caption("Flatman") 
		self.screen = pygame.display.get_surface()
		self.level = {}
		self.levelsize = 31
		self.level[0] = 0
		self.enemies = {}
		self.enemiesCount = 4
		self.player = Player(32*self.levelsize)
		self.small_point = pygame.image.load("small.png")
		self.large_point = pygame.image.load("large.png")
		self.large_point_anim = pygame.image.load("large_anim.png")
		self.resetGame()

	def resetGame(self):
		for i in range(1, self.levelsize):
			if i % 6 == 0:
				self.level[i] = 2
			else:
				self.level[i] = 1
		for i in range(self.enemiesCount):
			self.enemies[i] = Enemy(random.randint(5, self.levelsize), 32*(self.levelsize-1))
		self.player = Player(32*self.levelsize)
		self.points = 0
		self.reset = False
		self.gamestatus = "Running"
		self.pointAnim = False
		self.pointAnim_diff = 0
		
	def update(self, diff):
		for event in pygame.event.get():
			if event.type == QUIT:
				sys.exit(0)
			elif event.type == KEYDOWN:
				if event.key == 27: #esc
					sys.exit(0)
				elif event.key == 275: #right
					self.player.setDx(1)
				elif event.key == 276: #left
					self.player.setDx(-1)
				elif event.key == 114:
					self.reset = True
#				else:
#					print event.key
			elif event.type == KEYUP:
				if event.key == 275 or event.key == 276: #right or left
					self.player.setDx(0)
		self.player.update(diff)
		for i in range(self.enemiesCount):
			self.enemies[i].update(diff)
		ppos = max(0, int(self.player.getPos() / 32))
		if self.level[ppos] == 2:
			for i in range(self.enemiesCount):
				self.enemies[i].setEatable()
			self.level[ppos] = 0
			self.points = self.points + 5
		elif self.level[ppos] == 1:
			self.level[ppos] = 0
			if self.enemies[i].isEatable():
				self.points = self.points + 2
			else:
				self.points = self.points + 1

		for i in range(self.enemiesCount):
			epos = max(0, int(self.enemies[i].getPos() / 32))
			if epos == ppos and self.enemies[i].isAlive():
				if self.enemies[i].isEatable():
					self.enemies[i].setEaten()
					self.points = self.points + 10
				else:
					self.player.kill()
					self.gamestatus = "Lost"

		done = True
		for i in range(self.enemiesCount):
			if self.enemies[i].isAlive():
				done = False
		if done:
			for i in range(1, self.levelsize):
				if self.level[i] > 0:
					done = False
		if done:
			self.gamestatus = "Won"

		self.pointAnim_diff = self.pointAnim_diff + 1000 * diff
		if self.pointAnim_diff > 500:
			self.pointAnim_diff = 0
			self.pointAnim = not self.pointAnim

	def drawText(self, text, x, y):
		font = pygame.font.Font(None, 48)
		text = font.render(text, True, (255, 255, 255, 255))
		self.screen.blit(text, [x, y])
		
	def draw(self):
		self.screen.fill(pygame.Color(0,0,0,255))
		self.drawText("Points: " + str(self.points), 25, 25)
		self.drawText("Use cursors to move, press R to reset", 25, 75)
		if self.gamestatus != "Running":
			self.drawText(self.gamestatus, 25, 125)
		for i in range(self.levelsize):
			if self.level[i] == 2:
				if self.pointAnim:
					self.screen.blit(self.large_point, [32*i, 175])
				else:
					self.screen.blit(self.large_point_anim, [32*i, 175])
			elif self.level[i] == 1:
				self.screen.blit(self.small_point, [32*i, 175])
		for i in range(self.enemiesCount):
			self.enemies[i].draw(self.screen)
		self.player.draw(self.screen)
		pygame.display.flip()
		
	def gameLoop(self):
		lasttime = time.time()
		while True:
			now = time.time()
			diff = now - lasttime 
			self.update(diff)
			self.draw()
			if self.reset:
				self.resetGame()
			lasttime = now
			time.sleep(0.005)

game = Flatman()
game.gameLoop()